IF EXISTS (SELECT * FROM sysobjects WHERE id = object_id(N'[dbo].[VSE_Upgrade85To87]') AND OBJECTPROPERTY(id, N'IsProcedure') = 1)
DROP PROCEDURE [dbo].[VSE_Upgrade85To87]
GO

CREATE PROCEDURE dbo.VSE_Upgrade85To87
(
    @PolicySettingsID INT,
    @szPlatform NVARCHAR(50)
)
AS
BEGIN
    DECLARE @SectionName NVARCHAR(100)
    DECLARE @Value NVARCHAR(1024)

	/* ----------------------------------------------------------------------------
										[General]
										ScanProcessesOnEnable = 1
	---------------------------------------------------------------------------- */
    	SELECT @SectionName = @szPlatform + 'General'
     
    	IF EXISTS(SELECT *
          	FROM EPOPolicySettingValues 
          	WHERE EPOPolicySettingValues.PolicySettingsID = @PolicySettingsID
          	AND SectionName = @SectionName)
    	BEGIN
        	SELECT @Value = '1'
        	EXEC EPOPolicy_SetPolicySettingValue 0, @PolicySettingsID, @SectionName, 'ScanProcessesOnEnable', @Value
    	END

    
    	/* ----------------------------------------------------------------------------
	[EntExclItems]
	EnterceptExclusionModule_
	EnterceptExclusionAPI_			=>	EnterceptExclusionModule_.EnterceptExclusionAPI_
	---------------------------------------------------------------------------- */
	SELECT @SectionName = @szPlatform + 'EntExclItems'
	
	DECLARE @BOPExclusionCursor CURSOR;
	SET @BOPExclusionCursor = CURSOR FORWARD_ONLY READ_ONLY FOR
	SELECT SectionName, SettingName
	FROM EPOPolicySettingValues
	WHERE PolicySettingsID = @PolicySettingsID
	AND SectionName = @SectionName
	AND SettingName LIKE 'EnterceptExclusionProcess_%'
		
	OPEN @BOPExclusionCursor;
		
	DECLARE @PolicySectionName NVARCHAR(256)
	DECLARE @PolicySettingName NVARCHAR(256)
	DECLARE @PolicySettingValue NVARCHAR(1024)
	DECLARE	@PolicySettingValueCount NVARCHAR(10)
		    
	FETCH NEXT FROM @BOPExclusionCursor INTO @PolicySectionName, @PolicySettingName;
	WHILE (@@FETCH_STATUS = 0)
	BEGIN
		SELECT @PolicySettingValue = ''
		SELECT @Value = ''
	
		SELECT @PolicySettingValueCount = RIGHT (@PolicySettingName, LEN(@PolicySettingName) - CHARINDEX('_',@PolicySettingName))	
		
		SELECT @PolicySettingName = 'EnterceptExclusionModule_' + @PolicySettingValueCount		
		EXEC EPOPolicy_GetPolicySettingValue @PolicySettingsID, @PolicySectionName, @PolicySettingName, @PolicySettingValue OUTPUT
		
		SELECT @PolicySettingName = 'EnterceptExclusionAPI_' + @PolicySettingValueCount		        
		EXEC EPOPolicy_GetPolicySettingValue @PolicySettingsID, @PolicySectionName, @PolicySettingName, @Value OUTPUT
		
		IF @PolicySettingValue = ''
		OR @PolicySettingValue IS NULL
		BEGIN	
			SELECT @PolicySettingValue = '*'
		END
		
		IF @Value = ''
		OR @Value IS NULL
		BEGIN	
			SELECT @Value = '*'
		END
		
		SELECT @PolicySettingValue = @PolicySettingValue + '.' + @Value
	
		EXEC EPOPolicy_SetPolicySettingValue 0, @PolicySettingsID, @SectionName, @PolicySettingName, @PolicySettingValue
		
	FETCH NEXT FROM @BOPExclusionCursor INTO @PolicySectionName, @PolicySettingName;
	END
		    
	CLOSE @BOPExclusionCursor;
	DEALLOCATE @BOPExclusionCursor;    
END
GO    

IF EXISTS (SELECT * FROM sysobjects WHERE id = object_id(N'[dbo].[VSE_UpgradeEnforce]') AND OBJECTPROPERTY(id, N'IsProcedure') = 1)
DROP PROCEDURE [dbo].[VSE_UpgradeEnforce]
GO

CREATE PROCEDURE dbo.VSE_UpgradeEnforce
(
    @PolicySettingsID INT,
    @PolicySettingsTypeID NVARCHAR(256),
    @NewFeatureTextID NVARCHAR(50)
)
AS
BEGIN   
    DECLARE @PolicyObjectID INT
    SELECT @PolicyObjectID = EPOPolicyObjectToSettings.PolicyObjectID
    FROM EPOPolicyObjectToSettings
    WHERE EPOPolicyObjectToSettings.PolicySettingsID = @PolicySettingsID

    DECLARE @TypeTextID NVARCHAR(256)
    SELECT @TypeTextID = EPOPolicyTypes.TypeTextID
    FROM EPOPolicyTypes
    WHERE EPOPolicyTypes.TypeID = @PolicySettingsTypeID

    DECLARE @NewTypeID INT
    SELECT @NewTypeID = EPOPolicyTypes.TypeID
    FROM EPOPolicyTypes
    WHERE EPOPolicyTypes.TypeTextID = @TypeTextID
    AND EPOPolicyTypes.FeatureTextID = @NewFeatureTextID

    DECLARE @PolicyObjectName NVARCHAR(256)
    SELECT @PolicyObjectName = EPOPolicyObjects.Name
    FROM EPOPolicyObjects
    WHERE EPOPolicyObjects.PolicyObjectID = @PolicyObjectID

    DECLARE @NewPolicyObjectID INT
    SELECT @NewPolicyObjectID = EPOPolicyObjects.PolicyObjectID
    FROM EPOPolicyObjects
    WHERE EPOPolicyObjects.TypeID = @NewTypeID  
    AND EPOPolicyObjects.Name = @PolicyObjectName

    DECLARE @SlotID INT
    SELECT @SlotID = 0
    SELECT @SlotID = EPOPolicyAssignment.SlotID
    FROM EPOPolicyTypes, EPOPolicyObjects, EPOPolicyAssignment
    WHERE EPOPolicyTypes.TypeID = @NewTypeID
    AND EPOPolicyObjects.TypeID = EPOPolicyTypes.TypeID
    AND EPOPolicyAssignment.PolicyObjectID = EPOPolicyObjects.PolicyObjectID

    IF (@SlotID > 0)
    BEGIN
        DECLARE @AssignmentCursor CURSOR;
        SET @AssignmentCursor = CURSOR FORWARD_ONLY READ_ONLY FOR
            SELECT EPOPolicyAssignment.NodeID, EPOPolicyAssignment.NodeType, EPOPolicyAssignment.ForceInheritance, EPOPolicyAssignment.Hidden, EPOPolicyAssignment.TheTimestamp, EPOPolicyAssignment.TheHiddenTimestamp
            FROM EPOPolicyAssignment
            WHERE EPOPolicyAssignment.PolicyObjectID = @PolicyObjectID;
        
        OPEN @AssignmentCursor;
        
        DECLARE @NodeID INT
        DECLARE @NodeType INT
        DECLARE @ForceInheritance INT
        DECLARE @Hidden INT
        DECLARE @TheTimestamp INT
        DECLARE @TheHiddenTimestamp INT
        DECLARE @NewAssignmentID INT

        FETCH NEXT FROM @AssignmentCursor INTO @NodeID, @NodeType, @ForceInheritance, @Hidden, @TheTimestamp, @TheHiddenTimestamp;
        WHILE (@@FETCH_STATUS = 0)
        BEGIN   
            EXECUTE EPOPolicy_AssignPolicyToNode 0, @NewPolicyObjectID, @NodeID, @NodeType, @SlotID, @ForceInheritance, @Hidden, @NewAssignmentID
    
            FETCH NEXT FROM @AssignmentCursor INTO @NodeID, @NodeType, @ForceInheritance, @Hidden, @TheTimestamp, @TheHiddenTimestamp;
        END
    END 
    
    CLOSE @AssignmentCursor;
    DEALLOCATE @AssignmentCursor;
END
GO

IF EXISTS (SELECT * FROM sysobjects WHERE id = object_id(N'[dbo].[VSE_UpgradePolicy]') AND OBJECTPROPERTY(id, N'IsProcedure') = 1)
DROP PROCEDURE [dbo].[VSE_UpgradePolicy]
GO

CREATE PROCEDURE dbo.VSE_UpgradePolicy
(
    @PolicySettingsID INT,
    @PolicySettingsName NVARCHAR(256),
    @PolicySettingsTypeID NVARCHAR(256),
    @ProductCode NVARCHAR(50),
    @NewFeatureTextID NVARCHAR(50)
)
AS
BEGIN

    DECLARE @TypeTextID NVARCHAR(256)
    SELECT @TypeTextID = EPOPolicyTypes.TypeTextID
    FROM EPOPolicyTypes
    WHERE EPOPolicyTypes.TypeID = @PolicySettingsTypeID;

    DECLARE @PolicyObjectID INT
    SELECT @PolicyObjectID = EPOPolicyObjectToSettings.PolicyObjectID
    FROM EPOPolicyObjectToSettings
    WHERE EPOPolicyObjectToSettings.PolicySettingsID = @PolicySettingsID

    DECLARE @UserID INT
    SELECT @UserID = 0
    SELECT @UserID = UserID
    FROM EPOPolicyObjectUserRoles
    WHERE EPOPolicyObjectUserRoles.PolicyObjectID = @PolicyObjectID
    AND EPOPolicyObjectUserRoles.ACFlags = 1

    DECLARE @NewTypeID INT
    SELECT @NewTypeID = 0
    SELECT @NewTypeID = EPOPolicyTypes.TypeID
    FROM EPOPolicyTypes
    WHERE EPOPolicyTypes.TypeTextID = @TypeTextID
    AND EPOPolicyTypes.FeatureTextID = @NewFeatureTextID
    IF (@NewTypeID > 0)
    BEGIN
        IF (@PolicySettingsName LIKE 'My Default%') 
        BEGIN
            DELETE FROM EPOPolicyObjects
            WHERE EPOPolicyObjects.Name = @PolicySettingsName
            AND EPOPolicyObjects.TypeID = @NewTypeID
        END
        
        DECLARE @NewPolicyObjectID INT
        DECLARE @NewPolicySettingsName NVARCHAR(256)
        SELECT @NewPolicySettingsName = @PolicySettingsName + ' (VSE 8.7i)'         
        EXECUTE EPOPolicy_CopyPolicyObject 0, @PolicyObjectID, @NewPolicySettingsName, @UserID, 1, @NewPolicyObjectID OUTPUT    
        
        UPDATE EPOPolicyObjects
        SET TypeID = @NewTypeID
        WHERE PolicyObjectID = @NewPolicyObjectID           
        
        IF NOT EXISTS(SELECT * FROM EPOPolicyObjects 
                        WHERE @NewTypeID = TypeID 
                        AND @PolicySettingsName = Name)
        BEGIN
            UPDATE EPOPolicyObjects
            SET Name = @PolicySettingsName
            WHERE PolicyObjectID = @NewPolicyObjectID 
        END
    
        DECLARE @NewPolicySettingsID INT
        SELECT @NewPolicySettingsID = EPOPolicyObjectToSettings.PolicySettingsID
        FROM EPOPolicyObjectToSettings
        WHERE EPOPolicyObjectToSettings.PolicyObjectID = @NewPolicyObjectID 
    
        UPDATE EPOPolicySettings
        SET TypeID = @NewTypeID
        WHERE PolicySettingsID = @NewPolicySettingsID       

		IF @ProductCode = 'VIRUSCAN8600'
		BEGIN
			EXECUTE VSE_Upgrade85To87 @NewPolicySettingsID, 'Server_'
			EXECUTE VSE_Upgrade85To87 @NewPolicySettingsID, 'Wrkstn_'
		END

        DECLARE @SlotID INT
        SELECT @SlotID = 0
        SELECT @SlotID = EPOPolicyAssignment.SlotID
        FROM EPOPolicyTypes, EPOPolicyObjects, EPOPolicyAssignment
        WHERE EPOPolicyTypes.TypeID = @NewTypeID
        AND EPOPolicyObjects.TypeID = EPOPolicyTypes.TypeID
        AND EPOPolicyAssignment.PolicyObjectID = EPOPolicyObjects.PolicyObjectID
    
        IF (@SlotID > 0)
        BEGIN   
            DECLARE @AssignmentCursor CURSOR;
            SET @AssignmentCursor = CURSOR FORWARD_ONLY READ_ONLY FOR
                SELECT EPOPolicyAssignment.NodeID, EPOPolicyAssignment.NodeType, EPOPolicyAssignment.ForceInheritance, EPOPolicyAssignment.Hidden, EPOPolicyAssignment.TheTimestamp, EPOPolicyAssignment.TheHiddenTimestamp
                FROM EPOPolicyAssignment
                WHERE EPOPolicyAssignment.PolicyObjectID = @PolicyObjectID;
            
            OPEN @AssignmentCursor;
            
            DECLARE @NodeID INT
            DECLARE @NodeType INT
            DECLARE @ForceInheritance INT
            DECLARE @Hidden INT
            DECLARE @TheTimestamp INT
            DECLARE @TheHiddenTimestamp INT
            DECLARE @NewAssignmentID INT
            FETCH NEXT FROM @AssignmentCursor INTO @NodeID, @NodeType, @ForceInheritance, @Hidden, @TheTimestamp, @TheHiddenTimestamp;
            WHILE (@@FETCH_STATUS = 0)
            BEGIN
                EXECUTE EPOPolicy_AssignPolicyToNode 0, @NewPolicyObjectID, @NodeID, @NodeType, @SlotID, @ForceInheritance, @Hidden, @NewAssignmentID
    
                FETCH NEXT FROM @AssignmentCursor INTO @NodeID, @NodeType, @ForceInheritance, @Hidden, @TheTimestamp, @TheHiddenTimestamp;
            END
            
            CLOSE @AssignmentCursor;
            DEALLOCATE @AssignmentCursor;
        END
    END
END
GO

DECLARE @ProductCode NVARCHAR(30)

SELECT @ProductCode = ''

IF EXISTS (SELECT * FROM EPOPolicyTypes 
		WHERE FeatureTextID = 'VIRUSCAN8600')
BEGIN
	SELECT @ProductCode = 'VIRUSCAN8600'
END

IF @ProductCode <> ''
BEGIN
    DECLARE @PolicyCursor CURSOR;
    SET @PolicyCursor = CURSOR FORWARD_ONLY READ_ONLY FOR
        SELECT EPOPolicySettings.PolicySettingsID, EPOPolicyObjects.Name, EPOPolicySettings.TypeID
        FROM EPOPolicyProductToTypes, EPOPolicySettings, EPOPolicyObjects, EPOPolicyObjectToSettings
        WHERE EPOPolicyProductToTypes.TypeID = EPOPolicySettings.TypeID
        AND EPOPolicyProductToTypes.TypeID = EPOPolicyObjects.TypeID
        AND EPOPolicyObjectToSettings.PolicySettingsID =  EPOPolicySettings.PolicySettingsID
        AND EPOPolicyObjectToSettings.PolicyObjectID = EPOPolicyObjects.PolicyObjectID
        AND EPOPolicyProductToTypes.ProductCode = @ProductCode
        AND EPOPolicyProductToTypes.EditURL != 'dummy.htm'
        AND EPOPolicyObjects.Name != 'McAfee Default';
    
    OPEN @PolicyCursor;
    
    DECLARE @PolicySettingsID INT
    DECLARE @PolicySettingsName NVARCHAR(256)
    DECLARE @PolicySettingsTypeID NVARCHAR(256)
    
    FETCH NEXT FROM @PolicyCursor INTO @PolicySettingsID, @PolicySettingsName, @PolicySettingsTypeID;
    WHILE (@@FETCH_STATUS = 0)
    BEGIN
        EXECUTE VSE_UpgradePolicy @PolicySettingsID, @PolicySettingsName, @PolicySettingsTypeID, @ProductCode, 'VIRUSCAN8700'
    
        FETCH NEXT FROM @PolicyCursor INTO @PolicySettingsID, @PolicySettingsName, @PolicySettingsTypeID;
    END
    
    CLOSE @PolicyCursor;
    DEALLOCATE @PolicyCursor;


    DECLARE @EnforceCursor CURSOR;
    SET @EnforceCursor = CURSOR FORWARD_ONLY READ_ONLY FOR
        SELECT EPOPolicySettings.PolicySettingsID, EPOPolicySettings.TypeID
        FROM EPOPolicyProductToTypes, EPOPolicySettings, EPOPolicyObjects, EPOPolicyObjectToSettings
        WHERE EPOPolicyProductToTypes.TypeID = EPOPolicySettings.TypeID
        AND EPOPolicyProductToTypes.TypeID = EPOPolicyObjects.TypeID
        AND EPOPolicyObjectToSettings.PolicySettingsID =  EPOPolicySettings.PolicySettingsID
        AND EPOPolicyObjectToSettings.PolicyObjectID = EPOPolicyObjects.PolicyObjectID
        AND EPOPolicyProductToTypes.ProductCode = @ProductCode
        AND EPOPolicyProductToTypes.EditURL = 'dummy.htm'
        AND EPOPolicyObjects.Name != 'McAfee Default';
    
    OPEN @EnforceCursor;
            
    FETCH NEXT FROM @EnforceCursor INTO @PolicySettingsID, @PolicySettingsTypeID;
    WHILE (@@FETCH_STATUS = 0)
    BEGIN
        EXECUTE VSE_UpgradeEnforce @PolicySettingsID, @PolicySettingsTypeID, 'VIRUSCAN8700'
    
        FETCH NEXT FROM @EnforceCursor INTO @PolicySettingsID, @PolicySettingsTypeID;
    END
    
    CLOSE @EnforceCursor;
    DEALLOCATE @EnforceCursor;
    
    DECLARE @DefaultPolicySettingsID INT
            
	SELECT @DefaultPolicySettingsID = EPOPolicySettings.PolicySettingsID
	FROM EPOPolicySettings, EPOPolicyTypes
	WHERE EPOPolicyTypes.FeatureTextID LIKE 'VIRUSCAN8700'
	AND EPOPolicyTypes.TypeTextID = 'VSC700_BehaviorBlock_Policies'
	AND EPOPolicyTypes.TypeID = EPOPolicySettings.TypeID
	AND EPOPolicySettings.Name = 'McAfee Default'
	
	EXEC EPOPolicy_SetPolicySettingValue 0, @DefaultPolicySettingsID, 'ePOPolicyMigration', 'MigrationHappened', '1'    
END

IF EXISTS (SELECT * FROM sysobjects WHERE id = object_id(N'[dbo].[VSE_UpgradePolicy]') AND OBJECTPROPERTY(id, N'IsProcedure') = 1)
DROP PROCEDURE [dbo].[VSE_UpgradePolicy]
GO

IF EXISTS (SELECT * FROM sysobjects WHERE id = object_id(N'[dbo].[VSE_UpgradeEnforce]') AND OBJECTPROPERTY(id, N'IsProcedure') = 1)
DROP PROCEDURE [dbo].[VSE_UpgradeEnforce]
GO

IF EXISTS (SELECT * FROM sysobjects WHERE id = object_id(N'[dbo].[VSE_Upgrade85To87]') AND OBJECTPROPERTY(id, N'IsProcedure') = 1)
DROP PROCEDURE [dbo].[VSE_Upgrade85To87]
GO